package healthwatcher.aspects.exceptionHandling;

import healthwatcher.view.servlets.HWServlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.RemoteException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lib.exceptions.CommunicationException;
import lib.util.HTMLCode;

import org.aspectj.lang.SoftException;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.exceptionHandling.events.*;

/**
 * This aspect handles transaction exceptions in the servlets
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
@PrecedesHandler(healthwatcher.aspects.patterns.ServletCommanding.class)
public class HWDistributionExceptionHandler {

	public void handler(HWServletExEvent next) throws Throwable {
		
		PrintWriter out = next.response.getWriter();
		try {
			
			invoke(next);
			
		} catch (SoftException e) {
			if (e.getWrappedThrowable() instanceof CommunicationException) {
				out.println(HTMLCode.errorPage(e.getWrappedThrowable().getMessage()));
				out.close();
			} else if (e.getWrappedThrowable() instanceof RemoteException) {
				out.println(HTMLCode.errorPage("Comunication error, please try again later."));
				out.close();	
			} else {
				throw e;
	        }
		}
	}
	when HWServletExEvent do handler;
}
